__author__ = 'Dmitry Vasilyev'
__author_email__ = 'info@postgrespro.ru'
__description__ = 'Monitoring agent for PostgreSQL'
__version__ = '2.0.8'
__licence__ = 'BSD'

__url__ = 'https://github.com/postgrespro/mamonsu'

__classifiers__ = [
    'Environment :: Console',
    'Intended Audience :: System Administrators',
    'Operating System :: POSIX :: Linux',
    'Operating System :: Microsoft :: Windows',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Topic :: System :: Monitoring'
]
__keywords__ = [
    'monitoring',
    'zabbix',
    'postgres'
]
